@@ -104,8 +104,8 @@ typedef struct fifo {
 
 
 typedef void * TKEY;
-int GET_TKEY(TKEY);
-
+// int GET_TKEY(TKEY);
+#define GET_TKEY(x)	(x)
 
 void * new_block();
 void free_block();
