@@ -473,7 +473,7 @@ int			buf_alloc_size;
 	w = v->w;
 	h = v->h;
 
-	obj = d_alloc(sizeof(VIMAGE),316);
+	obj = d_alloc(sizeof(VIMAGE));
 	memset(obj, 0, sizeof(VIMAGE));
 	obj->vimage.ximage=NULL;
 	info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
@@ -494,15 +494,15 @@ int			buf_alloc_size;
 	buf_alloc_size = (win->d->gb2m_map.pixel_bits/8)*w*h;
 	switch(win->d->gb2m_map.pixel_bits){
 	case 32:
-		obj->vimage.buf_32 = (long*)d_alloc(buf_alloc_size, 317);
+		obj->vimage.buf_32 = (long*)d_alloc(buf_alloc_size);
 		memset(obj->vimage.buf_32, 0, buf_alloc_size);
 		break;
 	case 16:
-		obj->vimage.buf_16 = (short*)d_alloc(buf_alloc_size, 317);
+		obj->vimage.buf_16 = (short*)d_alloc(buf_alloc_size);
 		memset(obj->vimage.buf_16, 0, buf_alloc_size);
 		break;
 	case 8:
-		obj->vimage.buf_8 = (char*)d_alloc(buf_alloc_size, 317);
+		obj->vimage.buf_8 = (char*)d_alloc(buf_alloc_size);
 		memset(obj->vimage.buf_8, 0, buf_alloc_size);
 		break;
 	}
@@ -960,21 +960,21 @@ V_GN * v;
 		v->element_size = sizeof(long);
 		v->obj->vimage.buf_32 = d_alloc(sizeof(long)*
 				v->obj->vimage.w*
-				v->obj->vimage.h,125);
+				v->obj->vimage.h);
 	}
 	else if ( v->obj->vimage.buf_16 ) {
 		v->buf = v->obj->vimage.buf_16;
 		v->element_size = sizeof(short);
 		v->obj->vimage.buf_16 = d_alloc(sizeof(short)*
 				v->obj->vimage.w*
-				v->obj->vimage.h,125);
+				v->obj->vimage.h);
 	}
 	else if ( v->obj->vimage.buf_8 ) {
 		v->buf = v->obj->vimage.buf_8;
 		v->element_size = sizeof(char);
 		v->obj->vimage.buf_8 = d_alloc(sizeof(char)*
 				v->obj->vimage.w*
-				v->obj->vimage.h,125);
+				v->obj->vimage.h);
 	}
 	else {
 		er_panic("_v_get_and_new_image");
@@ -993,4 +993,4 @@ V_GN v;
     if ( element_size )
         *element_size = v.element_size;
     return v.buf;
-}
\ No newline at end of file
+}
