@@ -97,6 +97,80 @@ COORDINATE_UNIT * ret;
 	}
 }
 
+void
+setup_uenv(COORDINATE_UNIT * up,int ses)
+{
+int _ses;
+XL_SEXP * meta, * ret;
+XLISP_ENV * env;
+UNIT_ENV * ue;
+	_ses = 0;
+	if ( l_strcmp(up->system,l_string(std_cm,"base")) == 0 ) {
+		up->uenv = get_uenv(gblisp_top_env0);
+		return;
+	}
+	ue = search_uenv_name(up->system);
+	if ( ue ) {
+		up->uenv = ue;
+		return;
+	}
+	gc_push(0,0,"setup_uenv");
+	if ( ses < 0 ) {
+		ses = open_session(SEST_OPTIMIZE);
+		_ses = 1;
+	}
+	meta = load_meta(ses,"data",up->system);
+	switch ( get_type(meta) ) {
+	case XLT_ERROR:
+		gc_pop(0,0);
+		if ( _ses )
+			close_session(ses);
+		return;
+	case XLT_PAIR:
+		break;
+	default:
+		gc_pop(0,0);
+		if ( _ses )
+			close_session(ses);
+		return;
+	}
+	env = new_env(gblisp_top_env0);
+	for ( ; get_type(meta) == XLT_PAIR ; meta = cdr(meta) ) {
+		ret = eval(env,car(meta));
+	}
+	if ( env->e.uenv == 0 ) {
+		gc_pop(0,0);
+		if ( _ses )
+			close_session(ses);
+		return;
+	}
+	ue = env->e.uenv;
+	set_uenv_name(ue,up->system);
+	gc_pop(0,0);
+	if ( _ses )
+		close_session(ses);
+	up->uenv = ue;
+	return;
+}
+
+COORDINATE_UNIT *
+set_c_unit(COORDINATE_UNIT * up,L_CHAR * url,L_CHAR * system,L_CHAR * unit)
+{
+COORDINATE_UNIT * ret;
+	lock_task(cu_lock);
+	ret = _set_c_unit(url,system,unit);
+	if ( ret && up ) {
+		up->system = ll_copy_str(ret->system);
+		up->unit = ll_copy_str(ret->unit);
+		up->url = ll_copy_str(ret->url);
+		up->uenv = ret->uenv;
+	}
+	if ( up->uenv == 0 )
+		setup_uenv(up,-1);
+	unlock_task(cu_lock,"set_c_unit");
+	return ret;
+}
+
 void
 setup_c_unit(COORDINATE_UNIT * up,int ses,L_CHAR * url,
 	XL_SEXP * meta,XL_SYM_FIELD * sf)
@@ -108,30 +182,6 @@ UNIT_ENV * ue;
 XLISP_ENV * env;
 int _ses;
 
-
-retry:
-	_ses = 0;
-	up->uenv = 0;
-	lock_task(cu_lock);
-	u = _search_c_unit(url);
-	if ( u ) {
-		if ( u->system ) {
-			up->system = ll_copy_str(u->system);
-			up->unit = ll_copy_str(u->unit);
-			up->url = ll_copy_str(u->url);
-			unlock_task(cu_lock,"setup_c_unit");
-			goto next;
-		}
-		else {
-			sleep_task((int)u,cu_lock);
-			goto retry;
-		}
-	}
-	else {
-		_new_c_unit(url,0,0);
-	}
-	unlock_task(cu_lock,"setup_c_unit");
-
 	up->system = 0;
 	up->unit = 0;
 	up->url = 0;
@@ -145,6 +195,9 @@ retry:
 		meta = load_meta(ses,"meta",url);
 		switch ( get_type(meta) ) {
 		case XLT_ERROR:
+ss_printf("setup_c_unit error ");
+print_sexp(s_stdout,meta,0);
+ss_printf("\n");
 			gc_pop(0,0);
 			if ( _ses )
 				close_session(ses);
@@ -152,12 +205,40 @@ retry:
 		case XLT_PAIR:
 			break;
 		default:
+ss_printf("setup_c_unit error (type missmatch) ");
+print_sexp(s_stdout,meta,0);
+ss_printf("\n");
 			if ( _ses )
 				close_session(ses);
 			gc_pop(0,0);
 			return;
 		}
 	}
+
+retry:
+	_ses = 0;
+	up->uenv = 0;
+	lock_task(cu_lock);
+	u = _search_c_unit(url);
+	if ( u ) {
+		if ( u->system ) {
+			up->system = ll_copy_str(u->system);
+			up->unit = ll_copy_str(u->unit);
+			up->url = ll_copy_str(u->url);
+			unlock_task(cu_lock,"setup_c_unit");
+			gc_pop(0,0);
+			goto next;
+		}
+		else {
+			sleep_task((int)u,cu_lock);
+			goto retry;
+		}
+	}
+	else {
+		_new_c_unit(url,0,0);
+	}
+	unlock_task(cu_lock,"setup_c_unit");
+
 	if ( meta ) {
 		bu = car(meta);
 		if ( get_type(bu) != XLT_SYMBOL ||
