@@ -287,6 +287,7 @@ dm_resource_scan()
 RESOURCE * r;
 int i,j,len;
 DRAW_MATRIX * d;
+DRAW_MATRIX * d_st;
 int min,max;
 
 int target;
@@ -309,7 +310,7 @@ printf("scan %i\n",target);
 
 	for ( ; dm_nos > target ; ) {
 		lock_task(dm_lock);
-		d = R_PREV(DRAW_MATRIX *,&dm_h);
+		d_st = d = R_PREV(DRAW_MATRIX *,&dm_h);
 		for ( ; ; ) {
 			if ( d == (DRAW_MATRIX*)&dm_h )
 				d = R_PREV(DRAW_MATRIX*,&dm_h);
@@ -335,10 +336,15 @@ printf("scan %i\n",target);
 			break;
 		no:
 			d = R_PREV(DRAW_MATRIX*,&d->h);
+			if ( d == d_st )
+{ss_printf("UNRESOLVE SCAN DM\n");
+				goto end;
+}
 		}
 	}
 
 	lock_task(dm_lock);
+end:
 	dg_write_lock = 0;
 	wakeup_task((int)&dg_write_lock);
 	unlock_task(dm_lock,"scan");
