@@ -25,79 +25,139 @@
 #include	"xlerror.h"
 #include	"lock_level.h"
 #include	"pri_level.h"
+#include	"queue.h"
 
-
+/*
 typedef struct indicate_t {
 	Q_HEADER		h;
 	RESOURCE *		r;
 } INDICATE_T;
-
+*/
 
 #define IRQ_MAX		20
 
-extern GBVIEW_ENV env;
 extern RESOURCE ** res_entry_hash_table;
+extern SEM irq_lock,in_lock,iom_lock;
 
 void indicate_info_task();
 
-SEM indicate_lock,irq_lock;
-SEM iom_lock;
-int irq_cnt;
-int indicate_cnt;
-NESTED_LOCK * indicate_nlock;
-SYS_QUEUE	req_indicate;
-
 
 void
-init_indicate()
+gf_init_indicate(GBVIEW_FLAME * gf)
 {
-	indicate_lock = new_lock(LL_INDICATE);
-	irq_lock = new_lock(LL_IRQ);
-	iom_lock = new_lock(LL_ONMAP);
-	indicate_nlock = new_nlock();
-
-	memset(&req_indicate,0,sizeof(SYS_QUEUE));
-	req_indicate.flags = QF_FIFO;
-	req_indicate.gc_func = 0;
-	req_indicate.gc_get = 0;
-	req_indicate.key_func = 0;
-	req_indicate.pri = PRI_FETCH_STRONG;
-	setup_queue(&req_indicate);
-	
-	create_task(indicate_info_task,0,PRI_FETCH_STRONG);
+	ch_lock_init(&gf->indicate_lock);
+/*
+	memset(&gf->req_indicate,0,sizeof(SYS_QUEUE));
+	gf->req_indicate.flags = QF_FIFO;
+	gf->req_indicate.gc_func = 0;
+	gf->req_indicate.gc_get = 0;
+	gf->req_indicate.key_func = indicate_info_task;
+	gf->req_indicate.pri = PRI_FETCH_STRONG;
+	gf->req_indicate.work = gf;
+	setup_queue(&gf->req_indicate);
+*/
 }
 
 
+void
+xx_wf_indicate_lock(GBVIEW_FLAME * gf,char * file,int line)
+{
+	ch_lock(&gf->indicate_lock,file,line);
+}
 
+#define wf_indicate_lock(gf)	xx_wf_indicate_lock(gf,__FILE__,__LINE__)
+void
+wf_indicate_unlock(GBVIEW_FLAME * gf)
+{
+	ch_unlock(&gf->indicate_lock);
+}
 
+void
+wf_irq_lock(GBVIEW_FLAME * gf)
+{
+	lock_task(irq_lock);
+}
+
+void
+wf_irq_unlock(GBVIEW_FLAME * gf)
+{
+	unlock_task(irq_lock,"wf_irq_unlock");
+}
+
+void
+wf_iom_lock()
+{
+	lock_task(iom_lock);
+}
+
+void
+wf_iom_unlock()
+{
+	unlock_task(iom_lock,"iom");
+}
+
+void
+gc_indicate_func(GBVIEW_FLAME * gf)
+{
+INFO_LIST * il;
+	for ( il = gf->infolist ; il ; il = il->next )
+		gc_gb_sexp(il->info);
+}
 
 void
 gc_indicate()
 {
-INFO_LIST * il;
-	for ( il = env.infolist ; il ; il = il->next )
-		gc_gb_sexp(il->info);
+GBVIEW_FLAME * gf;
+extern GBVIEW_FLAME * gbview_flame_list;
+	for ( gf = gbview_flame_list ; gf ; gf = gf->next )
+		gc_indicate_func(gf);
 }
 
+void
+gc_indicate_lock()
+{
+GBVIEW_FLAME * gf;
+extern GBVIEW_FLAME * gbview_flame_list;
+extern SEM gbview_flame_lock;
+	if ( gbview_flame_lock == 0 )
+		return;
+	lock_task(gbview_flame_lock);
+	for ( gf = gbview_flame_list ; gf ; gf = gf->next ) {
+		_ch_lock(&gf->indicate_lock,__FILE__,__LINE__);
+	}
+}
 
+void
+gc_indicate_unlock()
+{
+GBVIEW_FLAME * gf;
+extern GBVIEW_FLAME * gbview_flame_list;
+extern SEM gbview_flame_lock;
+	if ( gbview_flame_lock == 0 )
+		return;
+	for ( gf = gbview_flame_list ; gf ; gf = gf->next ) {
+		_ch_unlock(&gf->indicate_lock);
+	}
+	unlock_task(gbview_flame_lock,"gc_indicate_unlock");
+}
 
 INDICATE *
-_search_indicate(URL * u)
+_search_indicate(GBVIEW_FLAME * gf,URL * u)
 {
 INDICATE * in;
-	for ( in = env.indicate ;
+	for ( in = gf->indicate ;
 		in &&
-		url_cmp(&in->url,u);
+		url_cmp_str(&in->url,u);
 		in = in->next );
 	return in;
 }
 
 void
-_delete_indicate(URL * u)
+_delete_indicate(GBVIEW_FLAME * gf,URL * u)
 {
 INDICATE ** inp,* in;
-	for ( inp = &env.indicate ; *inp ; inp = &(*inp)->next )
-		if ( url_cmp(&(*inp)->url,u) == 0 ) {
+	for ( inp = &gf->indicate ; *inp ; inp = &(*inp)->next )
+		if ( url_cmp_str(&(*inp)->url,u) == 0 ) {
 			in = *inp;
 			*inp = in->next;
 			free_url(&in->url);
@@ -107,41 +167,41 @@ INDICATE ** inp,* in;
 }
 
 INDICATE *
-_insert_indicate(URL * u)
+_insert_indicate(GBVIEW_FLAME * gf,URL * u)
 {
 INDICATE * in, ** inp;
-	in = _search_indicate(u);
+	in = _search_indicate(gf,u);
 	if ( in == 0 ) {
 		in = d_alloc(sizeof(*in));
 		copy_url(&in->url,u);
 		in->status = 0;
 		in->next = 0;
 		in->flags = 0;
-		for ( inp = &env.indicate ; *inp ; inp = &(*inp)->next );
+		for ( inp = &gf->indicate ; *inp ; inp = &(*inp)->next );
 		*inp = in;
 	}
 	return in;
 }
 
-
+/*
 void
-insert_indicate(URL * u)
+insert_indicate(GBVIEW_FLAME * gf,URL * u)
 {
 
-	nlock_task(indicate_nlock);
-	_insert_indicate(u);
-	unnlock_task(indicate_nlock);
+	nlock_task(gf->indicate_nlock);
+	_insert_indicate(gf,u);
+	unnlock_task(gf->indicate_nlock);
 
 	return;
 }
-
+*/
 
 INDICATE *
-_copy_indicate()
+_copy_indicate(GBVIEW_FLAME * gf)
 {
 INDICATE * ret, * in, * in1;
 	ret = 0;
-	for ( in = env.indicate ; in ; in = in->next ) {
+	for ( in = gf->indicate ; in ; in = in->next ) {
 		in1 = d_alloc(sizeof(*in1));
 		*in1 = *in;
 		copy_url(&in1->url,&in->url);
@@ -151,15 +211,17 @@ INDICATE * ret, * in, * in1;
 	return ret;
 }
 
+/*
 INDICATE *
-copy_indicate()
+copy_indicate(GBVIEW_FLAME * gf)
 {
 INDICATE * ret;
-	nlock_task(indicate_nlock);
-	ret = _copy_indicate();
-	unnlock_task(indicate_nlock);
+	nlock_task(gf->indicate_nlock);
+	ret = _copy_indicate(gf);
+	unnlock_task(gf->indicate_nlock);
 	return ret;
 }
+*/
 
 void
 free_indicate(INDICATE * in)
@@ -173,28 +235,31 @@ INDICATE * in1;
 	}
 }
 
+
 void
-set_pointer_indicate(int flags,URL * u)
+set_pointer_indicate(GBVIEW_FLAME * gf,int flags,URL * u)
 {
 INDICATE * in;
-	nlock_task(indicate_nlock);
+	if ( gf->status & GVFM_ZONBIE )
+		return;
+	lock_task(in_lock);
 	if ( flags ) {
-		in = _insert_indicate(u);
+		in = _insert_indicate(gf,u);
 		in->flags |= flags&(IF_COORDINATE|IF_OBJECT);
 	}
 	else {
-		_delete_indicate(u);
+		_delete_indicate(gf,u);
 	}
-	unnlock_task(indicate_nlock);
+	unlock_task(in_lock,"set_pointer_indicate");
 }
 
 
 void
-_clear_info()
+_clear_info(GBVIEW_FLAME * gf)
 {
 INFO_LIST * il_1, *il_2;
-	il_1 = env.infolist;
-	env.infolist = 0;
+	il_1 = gf->infolist;
+	gf->infolist = 0;
 	for ( ; il_1 ; ) {
 		il_2 = il_1;
 		il_1 = il_1->next;
@@ -203,19 +268,19 @@ INFO_LIST * il_1, *il_2;
 }
 
 void
-clear_info()
+clear_info(GBVIEW_FLAME * gf)
 {
-	lock_task(indicate_lock);
-	_clear_info();
-	unlock_task(indicate_lock,"clear_info");
+	wf_indicate_lock(gf);
+	_clear_info(gf);
+	wf_indicate_unlock(gf);
 }
 
 INFO_LIST *
-_copy_infolist()
+_copy_infolist(GBVIEW_FLAME * gf)
 {
 INFO_LIST * il_1,*il_2,* ret;
 	ret = 0;
-	for ( il_1 = env.infolist ; il_1 ; il_1 = il_1->next ) {
+	for ( il_1 = gf->infolist ; il_1 ; il_1 = il_1->next ) {
 		il_2 = d_alloc(sizeof(*il_2));
 		*il_2 = *il_1;
 		il_2->next = ret;
@@ -225,12 +290,12 @@ INFO_LIST * il_1,*il_2,* ret;
 }
 
 INFO_LIST *
-copy_infolist()
+copy_infolist(GBVIEW_FLAME * gf)
 {
 INFO_LIST * ret;
-	lock_task(indicate_lock);
-	ret = _copy_infolist();
-	unlock_task(indicate_lock,"copy_infolist");
+	wf_indicate_lock(gf);
+	ret = _copy_infolist(gf);
+	wf_indicate_unlock(gf);
 	return ret;
 }
 
@@ -244,13 +309,28 @@ INFO_LIST * il1;
 		d_f_ree(il1);
 	}
 }
+/*
+void
+free_req_indicate(SYS_QUEUE * que)
+{
+INDICATE_T * n;
+	for ( ; ; ) {
+		n = delete_queue(que,0,0,0);
+		if ( n == 0 )
+			break;
+		if ( n->h.key )
+			d_f_ree(n->h.key);
+		d_f_ree(n);
+	}	
+}
+*/
 
 void
-indicate_info_card(RESOURCE * r,OBJ * o)
+indicate_info_card(GBVIEW_FLAME * gf,RESOURCE * r,OBJ * o)
 {
 INFO_LIST * il;
 
-	lock_task(indicate_lock);
+	wf_indicate_lock(gf);
 	if ( (o->h.flags&OF_INFO_STS) == OF_INFO_FETCH &&
 			 o->h.info_org == 0 )
 		goto end;
@@ -260,6 +340,7 @@ INFO_LIST * il;
 	il->code = o->h.code;
 	il->r = r;
 	il->func = 0;
+	il->func_work = 0;
 	switch ( (o->h.flags & OF_INFO_STS)  ) {
 	case OF_INFO_FETCH:
 		il->info = o->h.info_card;
@@ -276,10 +357,10 @@ INFO_LIST * il;
 		il->flags |= OF_INFO_PROC;
 		break;
 	}
-	il->next = env.infolist;
-	env.infolist = il;
+	il->next = gf->infolist;
+	gf->infolist = il;
 end:
-	unlock_task(indicate_lock,"indicate_info_card");
+	wf_indicate_unlock(gf);
 }
 
 L_CHAR *
@@ -300,51 +381,58 @@ XL_SYM_FIELD * sf;
 	return 0;
 }
 
+typedef struct gf_status {
+	GBVIEW_FLAME * 		gf;
+	GBVIEW_STATUS		sts;
+} GF_STATUS;
 
 void
-_click_event_tick(GBVIEW_STATUS * sts)
+_click_event_tick(GF_STATUS * gfs)
 {
-	if ( env.event )
-		(*env.event)(ET_INFO,sts);
-	wf_free_status(sts);
-	d_f_ree(sts);
+	if ( gfs->gf->event )
+		(*gfs->gf->event)(gfs->gf,ET_INFO,&gfs->sts);
+	wf_free_status(&gfs->sts);
+	d_f_ree(gfs);
 }
 
 void
-click_info_url(L_CHAR * url)
+click_info_url(GBVIEW_FLAME * gf,L_CHAR * url)
 {
-GBVIEW_STATUS * sts;
-	sts = d_alloc(sizeof(*sts));
-	wf_init_status(sts);
-	sts->flags = SF_REFERENCE;
-	sts->reference = ll_copy_str(url);
-	new_tick(_click_event_tick,0,(int)sts);
+GF_STATUS * gfs;
+	gfs = d_alloc(sizeof(*gfs));
+	gfs->gf = gf;
+	wf_init_status(&gfs->sts);
+	gfs->sts.flags = SF_REFERENCE;
+	gfs->sts.reference = ll_copy_str(url);
+	new_tick(_click_event_tick,0,(int)gfs);
 }
 
 void
-click_infolist()
+click_infolist(GBVIEW_FLAME * gf)
 {
-GBVIEW_STATUS * sts;
-	sts = d_alloc(sizeof(*sts));
-	wf_init_status(sts);
-	sts->flags = SF_INFOLIST;
-	sts->infolist = _copy_infolist();
-	new_tick(_click_event_tick,0,(int)sts);
+GF_STATUS * gfs;
+	gfs = d_alloc(sizeof(*gfs));
+	gfs->gf = gf;
+	wf_init_status(&gfs->sts);
+	gfs->sts.flags = SF_INFOLIST;
+	gfs->sts.infolist = _copy_infolist(gf);
+	new_tick(_click_event_tick,0,(int)gfs);
 }
 
 void
-click_info_error()
+click_info_error(GBVIEW_FLAME * gf)
 {
-GBVIEW_STATUS * sts;
-	sts = d_alloc(sizeof(*sts));
-	wf_init_status(sts);
-	sts->flags = SF_ERROR;
-	new_tick(_click_event_tick,0,(int)sts);
+GF_STATUS * gfs;
+	gfs = d_alloc(sizeof(*gfs));
+	gfs->gf = gf;
+	wf_init_status(&gfs->sts);
+	gfs->sts.flags = SF_ERROR;
+	new_tick(_click_event_tick,0,(int)&gfs);
 }
 
 
 void
-ready_indicate()
+ready_indicate(GBVIEW_FLAME * gf)
 {
 INFO_LIST * il;
 XL_SEXP * s;
@@ -353,9 +441,9 @@ XL_SEXP * ss;
 L_CHAR * url;
 
 
-	if ( env.infolist == 0 )
+	if ( gf->infolist == 0 )
 		return; 
-	il = env.infolist;
+	il = gf->infolist;
 	i = 0;
 	for ( ; il ; il = il->next ) {
 		if ( (il->flags&OF_INFO_STS) != OF_INFO_FETCH ) {
@@ -363,55 +451,64 @@ L_CHAR * url;
 		}
 		i ++;
 	}
-	for ( il = env.infolist; il ; il = il->next ) {
+	for ( il = gf->infolist; il ; il = il->next ) {
 		if ( (il->flags&OF_INFO_SCHEME) == OF_INFO_URI ) {
 			if ( get_type(il->info) != XLT_PAIR ) {
-				click_info_error();
+				click_info_error(gf);
 				goto end;
 			}
 			ss = get_el(il->info,1);
 			if ( get_type(ss) != XLT_STRING ) {
-				click_info_error();
+				click_info_error(gf);
 				goto end;
 			}
-			click_info_url(ss->string.data);
+			click_info_url(gf,ss->string.data);
 			goto end;
 		}
 		url = get_info_url(il->obj->h.info_org);
 		if ( url ) {
-			click_info_url(url);
+			click_info_url(gf,url);
 			goto end;
 		}
 	}
-	click_infolist();
+	click_infolist(gf);
 end:
-	_clear_info();
+	_clear_info(gf);
+}
+
+void
+free_info_req_queue(INFO_REQ_QUEUE * q)
+{
+	d_f_ree(q->h.key);
+	d_f_ree(q);
 }
 
 
-
-
 void
-insert_irq(RESOURCE * r,int code,int cmd,WF_ID wfid)
+insert_irq(GBVIEW_FLAME * gf,RESOURCE * r,int code,int cmd,WF_ID wfid)
 {
 INFO_REQ_QUEUE * ir;
-INDICATE_T * it;
-	lock_task(irq_lock);
-	ir = d_alloc(sizeof(*ir));
+char irq[20];
+	ir = new_queue_node(sizeof(*ir));
 	ir->code = code;
 	ir->cmd = cmd;
 if ( wfid == 0 )
 printf("??????????????? insert_irq\n");
 	ir->wfid = wfid;
-	ir->next = r->draw_gb.ir_queue;
-	r->draw_gb.ir_queue = ir;
-	irq_cnt ++;
-	unlock_task(irq_lock,"insert_irq");
+	ir->gf = gf;
+	ir->r = r;
+	sprintf(irq,"gf%x",gf);
+	ir->h.key = nl_copy_str(std_cm,irq);
+	insert_queue(&r->draw_gb.ir_que,ir,0);
+}
 
-	it = d_alloc(sizeof(*it));
-	memset(it,0,sizeof(*it));
-	it->r = r;
-	insert_queue(&req_indicate,it,1);
+
+int
+ir_cmp_code(INFO_REQ_QUEUE * q,int * code)
+{
+	if ( q->code == *code )
+		return 0;
+	return -1;
 }
 
 void
@@ -420,64 +517,63 @@ delete_irq(int rno,int code)
 INFO_REQ_QUEUE * ir;
 RESOURCE * r;
 OBJ * o;
-	lock_task(irq_lock);
 	r = search_resource_by_no(rno);
 	if ( r == 0 )
 		return;
-	for ( ir = r->draw_gb.ir_queue;
-		ir ; ir = ir->next ) {
-		if ( ir->code == code ) {
-			ir->cmd = 0;
-			break;
-		}
-	}
-	unlock_task(irq_lock,"delete_irq");
+	ir = delete_queue(&r->draw_gb.ir_que,ir_cmp_code,&code,0);
+	if ( ir == 0 )
+		return;
+	free_info_req_queue(ir);
 	o = search_obj(r,code);
 	if ( o == 0 )
 		return;
+	if ( (o->h.flags&OF_INFO_STS) ==
+			OF_INFO_PROC ) {
+		o->h.flags &= ~OF_INFO_STS;
+		o->h.flags |= OF_INFO_NOCHECK;
+	}
 }
 
 
 int
-invoke_indicate(void (* func)())
+invoke_indicate(GBVIEW_FLAME * gf,void (* func)(),void * func_work)
 {
 INFO_LIST * il;
 int f;
-	lock_task(indicate_lock);
+	wf_indicate_lock(gf);
 	f = 0;
-	for ( il = env.infolist ; il ; il = il->next ) {
+	for ( il = gf->infolist ; il ; il = il->next ) {
 		if ( (il->flags&OF_INFO_STS) != OF_INFO_FETCH ) {
 			il->info = 0;
 			il->func = func;
-			insert_irq(il->r,il->code,IRQ_INDICATE_CARD,0);
-			wakeup_task((int)init_indicate);
+			il->func_work = func_work;
+			insert_irq(gf,il->r,il->code,IRQ_INDICATE_CARD,0);
 			f = -1;
 		}
 	}
 	if ( f )
 		goto end;
-	ready_indicate();
+	ready_indicate(gf);
 end:
-	unlock_task(indicate_lock,"invoke_indicate");
+	wf_indicate_unlock(gf);
 	return f;
 }
 
 void
-request_onmap_info(RESOURCE * r,OBJ * o,WF_ID wfid)
+request_onmap_info(GBVIEW_FLAME * gf,RESOURCE * r,OBJ * o,WF_ID wfid)
 {
-	lock_task(indicate_lock);
+	wf_indicate_lock(gf);
 	if ( (o->h.flags&OF_INFO_STS) == OF_INFO_NOCHECK ) {
 		o->h.flags &= ~OF_INFO_STS;
 		o->h.flags |= OF_INFO_PROC;
-		insert_irq(r,o->h.code,IRQ_INDICATE_ONMAP,wfid);
-		wakeup_task((int)init_indicate);
+		insert_irq(gf,r,o->h.code,IRQ_INDICATE_ONMAP,wfid);
 	}
-	unlock_task(indicate_lock,"req_onmap_info");
+	wf_indicate_unlock(gf);
 }
 
 
 void
-_indicate_onmap(int draw_flag,
+_indicate_onmap(GBVIEW_FLAME * gf,int draw_flag,
 	WF_ID wfid,RESOURCE * r,OBJ * o,GB_POINT ind_ptr)
 {
 GB_RECT rect;
@@ -495,9 +591,10 @@ int sub;
 			o->h.flags |= OF_INFO_FETCH;
 			goto fetch;
 		}
-		request_onmap_info(r,o,wfid);
+		request_onmap_info(gf,r,o,wfid);
 		rect.tl = rect.br = ind_ptr;
 		wf_insert_si(
+			gf,
 			ISI_INSERT,
 			wfid,
 			r->h.no,
@@ -516,6 +613,7 @@ int sub;
 		if ( o->h.info_org == 0 )
 			return;
 		if ( wf_insert_si(
+				gf,
 				ISI_INFO,
 				wfid,
 				r->h.no,
@@ -577,6 +675,7 @@ printf("\n");
 	width = pic.rect.br.x - pic.rect.tl.x;
 	height = pic.rect.br.y - pic.rect.tl.y;
 	if ( wf_space_check_si(
+			gf,
 			wfid,r->h.no,(o->h.code&MAIN_CODE)|INFO_SUBCODE,
 				0,&rect,10) == 0 )
 		goto ok;
@@ -584,6 +683,7 @@ printf("\n");
 	rect.br.y -= height;
 	ind_ptr.y -= height;
 	if ( wf_space_check_si(
+			gf,
 			wfid,r->h.no,(o->h.code&MAIN_CODE)|INFO_SUBCODE,
 				0,&rect,10) == 0 )
 		goto ok;
@@ -591,6 +691,7 @@ printf("\n");
 	rect.br.y += 2*height;
 	ind_ptr.y += 2*height;
 	if ( wf_space_check_si(
+			gf,
 			wfid,r->h.no,(o->h.code&MAIN_CODE)|INFO_SUBCODE,
 				0,&rect,10) == 0 )
 		goto ok;
@@ -608,6 +709,7 @@ ok:
 			flags = SIF_ANCOR;
 		else	flags = 0;
 		wf_insert_si(
+			gf,
 			ISI_INSERT,
 			wfid,
 			r->h.no,
@@ -625,20 +727,21 @@ ok:
 }
 
 void
-indicate_onmap(int draw_flag,
+indicate_onmap(GBVIEW_FLAME * gf,int draw_flag,
 	WF_ID wfid,RESOURCE * r,OBJ * o,GB_POINT ind_ptr)
 {
-	lock_task(iom_lock);
-	_indicate_onmap(draw_flag,wfid,r,o,ind_ptr);
-	unlock_task(iom_lock,"indicate_onmap");
+	wf_iom_lock();
+	_indicate_onmap(gf,draw_flag,wfid,r,o,ind_ptr);
+	wf_iom_unlock();
 }
 
 void
-insert_indicate_info(RESOURCE * r,int code,int flags,XL_SEXP * data)
+insert_indicate_info(GBVIEW_FLAME * gf,
+	RESOURCE * r,int code,int flags,XL_SEXP * data)
 {
 INFO_LIST* il;
-	lock_task(indicate_lock);
-	il = env.infolist;
+	wf_indicate_lock(gf);
+	il = gf->infolist;
 	for ( ; il ; il = il->next ) {
 		if ( il->r != r )
 			continue;
@@ -647,10 +750,10 @@ INFO_LIST* il;
 		il->info = data;
 		il->flags = (flags&(~OF_INFO_STS))|OF_INFO_FETCH;
 		if ( il->func )
-			(*il->func)();
+			(*il->func)(il->func_work);
 		break;
 	}
-	unlock_task(indicate_lock,"insert_indicate_info");
+	wf_indicate_unlock(gf);
 }
 
 void
@@ -711,7 +814,7 @@ XL_SEXP * sym;
 }
 
 int
-setup_info(WF_ID wfid,RESOURCE * r,XL_SEXP * info)
+setup_info(GBVIEW_FLAME * gf,WF_ID wfid,RESOURCE * r,XL_SEXP * info)
 {
 XL_SEXP * s, * t, * u, * v;
 XL_SYM_FIELD * sf;
@@ -831,19 +934,19 @@ char * err_buf;
 			formating_info(r,o);
 			card_data = o->h.info_card;
 
-			lock_task(iom_lock);
+			wf_iom_lock();
 			o->h.flags &= ~OF_INFO_STS;
 			o->h.flags |= OF_INFO_FETCH;
 			if ( o->h.flags & OF_ONMAP_REQ ) {
 			GB_POINT ptr;
 				o->h.flags &= ~OF_ONMAP_REQ;
 				ptr.x = ptr.y = 0;
-				_indicate_onmap(0,wfid,r,o,ptr);
+				_indicate_onmap(gf,0,wfid,r,o,ptr);
 				ind = 1;
 			}
-			unlock_task(iom_lock,"setup_info");
+			wf_iom_unlock();
 		}
-		insert_indicate_info(r,code,flags,card_data);
+		insert_indicate_info(gf,r,code,flags,card_data);
 	}
 /*
 	lock_task(indicate_lock);
@@ -855,7 +958,7 @@ char * err_buf;
 }
 
 XL_SEXP *
-setup_info_cmd(WF_ID wfid,RESOURCE * r,int code,XL_SEXP * info)
+setup_info_cmd(GBVIEW_FLAME * gf,WF_ID wfid,RESOURCE * r,int code,XL_SEXP * info)
 {
 XL_SEXP  * v;
 XL_SYM_FIELD * sf;
@@ -906,7 +1009,7 @@ XL_SEXP * card_data;
 		formating_info(r,o);
 		card_data = o->h.info_card;
 
-		lock_task(iom_lock);
+		wf_iom_lock();
 		o->h.flags &= ~OF_INFO_STS;
 		o->h.flags |= OF_INFO_FETCH;
 		if ( o->h.flags & OF_ONMAP_REQ ) {
@@ -914,11 +1017,12 @@ XL_SEXP * card_data;
 			o->h.flags &= ~OF_ONMAP_REQ;
 			ptr.x = ptr.y = 0;
 			if ( wfid )
-				_indicate_onmap(0,wfid,r,o,ptr);
+				_indicate_onmap(gf,0,wfid,r,o,ptr);
 		}
-		unlock_task(iom_lock,"setup_info");
+		wf_iom_unlock();
 	}
-	insert_indicate_info(r,code,flags,card_data);
+	if ( gf )
+		insert_indicate_info(gf,r,code,flags,card_data);
 /*
 	lock_task(indicate_lock);
 	ready_indicate();
@@ -943,12 +1047,29 @@ no_obj:
 			-1));
 }
 
+typedef struct check_irq_t {
+	GBVIEW_FLAME * 	gf;
+	WF_ID		wfid;
+} CHECK_IRQ_T;
 
 int
-check_irq(int ses,RESOURCE * r)
+cmp_check_irq(SYS_QUEUE * q,INFO_REQ_QUEUE * ir,CHECK_IRQ_T *w)
+{
+	if ( ir->gf != w->gf )
+		return -1;
+	if ( w->wfid == 0 )
+		return 0;
+	if ( ir->wfid == w->wfid )
+		return 0;
+	return -1;
+}
+
+
+int
+check_irq(int ses,GBVIEW_FLAME * gf,RESOURCE * r)
 {
 XL_SEXP * query, * code;
-INFO_REQ_QUEUE * ir1, * ir2;
+INFO_REQ_QUEUE * ir2;
 XL_SEXP * ret,* all;
 int er;
 XL_SEXP * gvr;
@@ -960,22 +1081,25 @@ int req;
 OBJ * o;
 L_CHAR * f;
 int setup_ret;
+CHECK_IRQ_T w;
+int wait_flag;
 
-	lock_task(indicate_lock);
-	lock_task(irq_lock);
-	if ( r->draw_gb.ir_queue == 0 ) {
-		unlock_task(irq_lock,"check_irq");
-		unlock_task(indicate_lock,"check_irq");
-		return 0;
-	}
-	ir2 = ir1 = r->draw_gb.ir_queue;
+	wf_lock(gf);
+	ZONBIE_CHECK(gf,wf_unlock,1)
+	wf_unlock(gf);
 	gc_push(0,0,"check_irq");
 	query = 0;
 	req = 0;
-	for ( cnt = 0 ; ir1 && cnt < IRQ_MAX ; cnt ++ ) {
-		irq_cnt --;
-		ir2 = ir1;
-		ir1 = ir1->next;
+	w.gf = gf;
+	w.wfid = 0;
+
+	wait_flag = -5;
+	for ( cnt = 0 ; cnt < IRQ_MAX ; cnt ++ ) {
+		ir2 = delete_queue(&r->draw_gb.ir_que,cmp_check_irq,&w,
+					wait_flag);
+		wait_flag = 0;
+		if ( ir2 == 0 )
+			break;
 		if ( ir2->cmd ) {
 			code = get_integer(ir2->code,0);
 			query = cons(code,query);
@@ -989,14 +1113,13 @@ int setup_ret;
 				o->h.flags |= OF_INFO_NOCHECK;
 			}
 		}
-		wfid = ir2->wfid;
-		d_f_ree(ir2);
+		w.wfid = wfid = ir2->wfid;
+		free_info_req_queue(ir2);
+	}
+	if ( req == 0 ) {
+		gc_pop(0,0);
+		return 1;
 	}
-	r->draw_gb.ir_queue = ir1;
-	unlock_task(irq_lock,"check_irq");
-	unlock_task(indicate_lock,"check_irq");
-	if ( req == 0 )
-		return 0;
 
 	query = cons(
 		get_string(f=get_url_filepath(&r->h.target)),
@@ -1059,45 +1182,76 @@ os_test2 = 0;
 		er = -1;
 		goto end;
 	}
-	lock_pdb_lock(r,1);
-	setup_ret = setup_info(wfid,r,ret);
+	lock_pdb_lock(r,0);
+	setup_ret = setup_info(gf,wfid,r,ret);
 	unlock_pdb_lock(r);
 	if ( setup_ret )
-		_wf_set_redraw();
+		_wf_set_redraw(gf);
 end:
 	gc_pop(0,0);
 	return er;
 }
 
+typedef struct indicate_info_task_check_t {
+	GBVIEW_FLAME *		gf;
+	L_CHAR *		key;
+} INDICATE_INFO_TASK_CHECK_T;
+
+int
+indicate_info_task_check(
+	SYS_QUEUE * q,
+	INFO_REQ_QUEUE * ir,
+	INDICATE_INFO_TASK_CHECK_T * w)
+{
+	if ( w->gf )
+		return -1;
+	if ( l_strcmp(ir->h.key,w->key) )
+		return -1;
+	w->gf = ir->gf;
+	return 0;
+}
+
 
 void
-indicate_info_task()
+indicate_info_task(TKEY d)
 {
 XL_INTERPRETER * xli;
 int ses;
 int i;
 int f;
-INDICATE_T * it;
-
-printf("indicate task ****  %i\n",get_tid());
+L_CHAR * key;
+GBVIEW_FLAME * gf;
+SYS_QUEUE * que;
+RESOURCE * r;
+INDICATE_INFO_TASK_CHECK_T w;
 
 set_cpu_thr_type(5);
 
 
+	que = (SYS_QUEUE*)GET_TKEY(d);
+	key = touch_qkey(que);
+	r = que->work;
+	w.gf = 0;
+	w.key = key;
+	check_queue(que,indicate_info_task_check,&w);
+	if ( w.gf == 0 )
+		goto end;
+	gf = w.gf;
+
 	xli = new_xl_interpreter();
 	xli->a_type = XLA_SELF;
 	setup_i(xli);
 	ses = open_session(SEST_OPTIMIZE);
 
 	for ( ; ; ) {
-		it = delete_queue(&req_indicate,0,0,1);
-		if ( it == 0 )
-			continue;
-		if ( it->r->h.type != RT_DRAW_GB )
-			er_panic("indicate_info_task");
-		check_irq(ses,it->r);
-		d_f_ree(it);
+		if ( check_irq(ses,gf,r) == 1 )
+			break;
 	}
+
+	close_session(ses);
+	close_self_interpreter();
+end:
+	release_qkey(que,key);
 }
 
 
