@@ -23,7 +23,6 @@
 
 #define PRI_OFFSET_RATE		20
 
-extern GBVIEW_ENV env;
 void ov_default();
 
 
@@ -40,6 +39,7 @@ set_weight(WIN_FLAME * wf)
 int
 e2d_new(
 	WIN_FLAME_TABLE * tbl,
+	GBVIEW_FLAME * gf,
 	WF_ID *		ret_wfid,
 	REAL1		v_reso,
 	RESOURCE * 	draw,
@@ -60,19 +60,19 @@ int pri;
 */
 retry:
 	*ret_wfid = 0;
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 set_t_msg(55);
 
 
-	if ( env.flame_base && env.flame_base->flags & WFF_FREE ) {
-		sleep_task((int)env.flame_base,env.wf_lock);
+	if ( gf->flame_base && gf->flame_base->flags & WFF_FREE ) {
+		wf_sleep_task(gf,(int)gf->flame_base);
 		goto retry;
 	}
-	for ( wf = env.flame; wf ; wf = wf->next ) {
+	for ( wf = gf->flame; wf ; wf = wf->next ) {
 		if ( wf->draw != draw )
 			continue;
 		if ( wf->flags & WFF_FREE ) {
-			sleep_task((int)wf,env.wf_lock);
+			wf_sleep_task(gf,(int)wf);
 			goto retry;
 		}
 		wf->flags |= WFF_ACTIVE;
@@ -81,7 +81,7 @@ set_t_msg(55);
 	}
 	wf = d_alloc(sizeof(*wf));
 	wf->pri_adj = 0;
-	wf->id = _get_new_wfid();
+	wf->id = _get_new_wfid(gf);
 	wf->tbl = tbl;
 	wf->sym_i = 0;
 	wf->draw = draw;
@@ -91,21 +91,21 @@ set_t_msg(55);
 /*
 	draw->c.weight = 0;
 */
-	wf->pixels = d_alloc(env.win_width * env.win_height * sizeof(long));
-	for ( i = 0 ; i < env.win_width * env.win_height ; i ++ )
+	wf->pixels = d_alloc(gf->win_width * gf->win_height * sizeof(long));
+	for ( i = 0 ; i < gf->win_width * gf->win_height ; i ++ )
 		wf->pixels[i] = C_TRANSPARENT|C_DIRTY;
-	if ( env.flame_base == 0  ) {
-		env.flame_base = wf;
-		env.flame_base_rotate = rotate;
-		env.flame_base_resolution = resolution;
-		env.flame_base_center = center;
-		env.flame_base_display_map.dir = MHD_REVERSE;
-		env.flame_base_display_map.type = MHT_LINEAR;
-		env.flame_base_display_map.next = 0;
-		calc_pitch(1);
+	if ( gf->flame_base == 0  ) {
+		gf->flame_base = wf;
+		gf->flame_base_rotate = rotate;
+		gf->flame_base_resolution = resolution;
+		gf->flame_base_center = center;
+		gf->flame_base_display_map.dir = MHD_REVERSE;
+		gf->flame_base_display_map.type = MHT_LINEAR;
+		gf->flame_base_display_map.next = 0;
+		calc_pitch(gf,1);
 	}
 
-	for ( wfp = &env.flame ; *wfp ; wfp = &(*wfp)->next ) {
+	for ( wfp = &gf->flame ; *wfp ; wfp = &(*wfp)->next ) {
 		if ( (*wfp)->flags & WFF_FREE )
 			continue;
 		if ( (*wfp)->v_reso > v_reso )
@@ -117,14 +117,16 @@ set_t_msg(55);
 	wf->lock_list = 0;
 	wf->browse = 0;
 	wf->aboat_count = 0;
+	wf->in_base_time = get_xltime();
+	wf->out_base_time = 0;
 
 	wf->insert_time = get_xltime();
 
-	_wakeup_wf(wf);
+	_wakeup_wf(gf,wf);
 end:
 	*ret_wfid = wf->id;
-	assert_ls(LSF_ZOOM);
-	unlock_task(env.wf_lock,"new_win_flame");
+	assert_ls(gf,LSF_ZOOM);
+	wf_unlock(gf);
 	call_gv_event(RID_INDICATED_COORD,l_string(std_cm,"indicated"));
 	call_gv_event(RID_HIDE_COORD,l_string(std_cm,"indicated"));
 
@@ -133,7 +135,7 @@ end:
 
 
 int
-e2d_exit(WIN_FLAME * wf)
+e2d_exit(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 WIN_FLAME ** wfp;
 RESOURCE * r;
@@ -142,72 +144,71 @@ RESOURCE * r;
 	if ( !(wf->flags & WFF_FREE) ) {
 		return 0;
 	}
-	if ( env.flame_base == wf ) {
-		if ( _wf_set_current(wf,0) )
-			assert_ls(LSF_CURRENT);
+	if ( gf->flame_base == wf ) {
+		if ( _wf_set_current(gf,wf,0) )
+			assert_ls(gf,LSF_CURRENT);
 		else {
-			env.flame_base = 0;
+			gf->flame_base = 0;
 		}
 	}
 	free_mh(wf->mh);
 	d_f_ree(wf->pixels);
 	wf_free_si(wf);
-	for ( wfp = &env.flame ; *wfp != wf ; wfp = &(*wfp)->next );
+	for ( wfp = &gf->flame ; *wfp != wf ; wfp = &(*wfp)->next );
 	if ( *wfp )
 		*wfp = wf->next;
 	wakeup_task((int)wf);
 	wf_free_lock_list(wf);
 	d_f_ree(wf);
-	_wf_flame_redraw();
+	_wf_flame_redraw(gf);
 	call_gv_event(RID_INDICATED_COORD,l_string(std_cm,"hide"));
 	call_gv_event(RID_HIDE_COORD,l_string(std_cm,"hide"));
-	wakeup_task((int)&env.flame);
 	return 1;
 }
 
 
 void
-wf_move_x(WIN_FLAME * wf,int dx,GB_RECT * r)
+wf_move_x(GBVIEW_FLAME * gf,WIN_FLAME * wf,int dx,GB_RECT * r)
 {
 int x,y;
 unsigned long * dest, * src;
 	if ( dx == 0 )
 		return;
 	if ( dx > 0 ) {
-		for ( y = 0 ; y < env.win_height ; y ++ ) {
-			dest = &wf->pixels[(y+1)*env.win_width];
-			src = &wf->pixels[(y+1)*env.win_width - dx];
-			for ( x = env.win_width-dx ; x > 0 ; x -- )
+		for ( y = 0 ; y < gf->win_height ; y ++ ) {
+			dest = &wf->pixels[(y+1)*gf->win_width];
+			src = &wf->pixels[(y+1)*gf->win_width - dx];
+			for ( x = gf->win_width-dx ; x > 0 ; x -- )
 				*--dest = *--src;
-			dest = &wf->pixels[y*env.win_width];
+			dest = &wf->pixels[y*gf->win_width];
 			for ( x = dx ; x > 0 ; x -- )
 				*dest++ = C_TRANSPARENT|C_DIRTY;
 		}
 		r->tl.x = 0;
 		r->tl.y = 0;
 		r->br.x = dx;
-		r->br.y = env.win_height;
+		r->br.y = gf->win_height;
 	}
 	else {
-		for ( y = 0 ; y < env.win_height ; y ++ ) {
-			dest = &wf->pixels[y*env.win_width];
-			src = &wf->pixels[y*env.win_width- dx];
-			for ( x = env.win_width+dx ; x > 0 ; x -- )
+		for ( y = 0 ; y < gf->win_height ; y ++ ) {
+			dest = &wf->pixels[y*gf->win_width];
+			src = &wf->pixels[y*gf->win_width- dx];
+			for ( x = gf->win_width+dx ; x > 0 ; x -- )
 				*dest ++ = *src ++;
-			dest = &wf->pixels[(y+1)*env.win_width + dx];
+			dest = &wf->pixels[(y+1)*gf->win_width + dx];
 			for ( x = - dx ; x > 0 ; x -- )
 				*dest++ = C_TRANSPARENT| C_DIRTY;
 		}
-		r->tl.x = env.win_width+dx;
+		r->tl.x = gf->win_width+dx;
 		r->tl.y = 0;
-		r->br.x = env.win_width;
-		r->br.y = env.win_height;
+		r->br.x = gf->win_width;
+		r->br.y = gf->win_height;
 	}
 	wf->mov_x += dx;
 }
 
 void
-wf_move_y(WIN_FLAME * wf,int dy,GB_RECT * r)
+wf_move_y(GBVIEW_FLAME * gf,WIN_FLAME * wf,int dy,GB_RECT * r)
 {
 int x,y;
 unsigned long * dest, * src;
@@ -215,42 +216,42 @@ unsigned long * dest, * src;
 	if ( dy == 0 )
 		return;
 	if ( dy > 0 ) {
-		for ( y = env.win_height-1 ; y-dy >= 0 ; y -- ) {
-			dest = &wf->pixels[y*env.win_width];
-			src = &wf->pixels[(y-dy)*env.win_width];
-			for ( x = 0 ; x < env.win_width ; x ++ )
+		for ( y = gf->win_height-1 ; y-dy >= 0 ; y -- ) {
+			dest = &wf->pixels[y*gf->win_width];
+			src = &wf->pixels[(y-dy)*gf->win_width];
+			for ( x = 0 ; x < gf->win_width ; x ++ )
 				*dest++ = *src++;
 		}
 		dest = wf->pixels;
 		for ( y = 0 ; y < dy ; y ++ )
-			for ( x = 0 ; x < env.win_width ; x ++ )
+			for ( x = 0 ; x < gf->win_width ; x ++ )
 				*dest++ = C_TRANSPARENT|C_DIRTY;
 		r->tl.x = 0;
 		r->tl.y = 0;
-		r->br.x = env.win_width;
+		r->br.x = gf->win_width;
 		r->br.y = dy;
 	}
 	else {
-		for ( y = 0 ; y < env.win_height + dy ; y ++ ) {
-			src = &wf->pixels[(y-dy)*env.win_width];
-			dest = &wf->pixels[y*env.win_width];
-			for ( x = 0 ; x < env.win_width ; x ++ )
+		for ( y = 0 ; y < gf->win_height + dy ; y ++ ) {
+			src = &wf->pixels[(y-dy)*gf->win_width];
+			dest = &wf->pixels[y*gf->win_width];
+			for ( x = 0 ; x < gf->win_width ; x ++ )
 				*dest++ = *src++;
 		}
-		dest = &wf->pixels[(env.win_height+dy)*env.win_width];
+		dest = &wf->pixels[(gf->win_height+dy)*gf->win_width];
 		for ( y = 0 ; y < - dy ; y ++ )
-			for ( x = 0 ; x < env.win_width ; x ++ )
+			for ( x = 0 ; x < gf->win_width ; x ++ )
 				*dest ++ = C_TRANSPARENT|C_DIRTY;
 		r->tl.x = 0;
-		r->tl.y = env.win_height+dy;
-		r->br.x = env.win_width;
-		r->br.y = env.win_height;
+		r->tl.y = gf->win_height+dy;
+		r->br.x = gf->win_width;
+		r->br.y = gf->win_height;
 	}
 	wf->mov_y += dy;
 }
 
 int
-move_redraw_rect(WIN_FLAME * wf,GB_RECT *r)
+move_redraw_rect(GBVIEW_FLAME * gf,WIN_FLAME * wf,GB_RECT *r)
 {
 int min_x,max_x;
 int min_y,max_y;
@@ -287,34 +288,36 @@ int id;
 		for ( xx = r->tl.x ; xx < r->br.x ; xx += lp ) {
 			pt_list[i].x = xx;
 			pt_list[i].y = yy;
-			pt_reso[i] = env.flame_base_resolution;
+			pt_reso[i] = gf->flame_base_resolution;
 			buf[i] = C_TRANSPARENT;
 			i ++;
 		}
 	length = i;
 
 
-	mh = add_mh(&env.flame_base_display_map,wf->mh);
+	mh = add_mh(&gf->flame_base_display_map,wf->mh);
 
 	map_from_flame(mh,pt_list,pt_reso,length);
 
-	set_surp(&surp,r,env.flame_base_resolution);
+	set_surp(&surp,r,gf->flame_base_resolution);
 
 	map_from_flame(mh,surp.ptr,surp.reso,SURP_NOS);
 
 	id = wf->id;
 
-	_ex_lock();
+	_ex_lock(gf);
 
-	unlock_task(env.wf_lock,"move_rect");
-	cache_image(wf->id,wf->draw,pt_list,pt_reso,buf,length,r,mh,&surp,0,
+	wf_unlock(gf);
+	cache_image(
+		gf,
+		wf->id,wf->draw,pt_list,pt_reso,buf,length,r,mh,&surp,0,
 		WFF_LUSTER_DIRTY);
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 
-	_ex_unlock();
+	_ex_unlock(gf);
 set_t_msg(56);
 
-	_wf = _get_wf_ptr(id);
+	_wf = _get_wf_ptr(gf,id);
 	if ( _wf != wf )
 		goto end;
 
@@ -331,11 +334,11 @@ set_t_msg(56);
 				for ( dy = 0 ; dy < lp_y ; dy ++ ) {
 					if ( pt_reso[i] < 0 ) {
 						wf->pixels[xx+dx+
-							(yy+dy)*env.win_width]
+							(yy+dy)*gf->win_width]
 						|= C_DIRTY;
 						continue;
 					}
-					wf->pixels[xx+dx+(yy+dy)*env.win_width]
+					wf->pixels[xx+dx+(yy+dy)*gf->win_width]
 						= buf[i]|C_DIRTY;
 				}
 			i ++;
@@ -350,13 +353,13 @@ end:
 
 
 void
-wf_move_si(WIN_FLAME * wf,int dx,int dy)
+wf_move_si(GBVIEW_FLAME * gf,WIN_FLAME * wf,int dx,int dy)
 {
 GB_RECT r;
 SYMBOL_INDICATE ** sip, * si1;
 	r.tl.x = r.tl.y = 0;
-	r.br.x = env.win_width;
-	r.br.y = env.win_height;
+	r.br.x = gf->win_width;
+	r.br.y = gf->win_height;
 	for ( sip = &wf->sym_i ; *sip ; ) {
 		si1 = *sip;
 		si1->rect.tl.x += dx;
@@ -381,23 +384,24 @@ SYMBOL_INDICATE ** sip, * si1;
 
 
 int
-e2d_move(WIN_FLAME * wf,VPOINT from,VPOINT to,VPOINT * ix)
+e2d_move(GBVIEW_FLAME * gf,WIN_FLAME * wf,VPOINT from,VPOINT to,
+	 void * _ix)
 {
-extern int null_wf_spiral;
+VPOINT * ix;
 GB_RECT r1,r2;
 int dx,dy;
+	ix = _ix;
 	dx = to.x - from.x;
 	dy = to.y - from.y;
-	wf_move_x(wf,dx,&r1);
-	wf_move_y(wf,dy,&r2);
-	wf_move_si(wf,dx,dy);
+	wf_move_x(gf,wf,dx,&r1);
+	wf_move_y(gf,wf,dy,&r2);
+	wf_move_si(gf,wf,dx,dy);
 	if ( wf->flags & WFF_LUSTER ) {
 		if ( dx )
-			move_redraw_rect(wf,&r1);
+			move_redraw_rect(gf,wf,&r1);
 		if ( dy )
-			move_redraw_rect(wf,&r2);
+			move_redraw_rect(gf,wf,&r2);
 	}
-	null_wf_spiral = 1;
 	if ( wf->flags & WFF_LUSTER )
 		wf->flags |= WFF_LUSTER_DIRTY;
 	if ( wf->flags & WFF_POLY )
@@ -411,6 +415,7 @@ int dx,dy;
 int
 e2d_change(
 	WIN_FLAME_TABLE * tbl,
+	GBVIEW_FLAME * gf,
 	WIN_FLAME * wf)
 {
 int i;
@@ -418,9 +423,9 @@ int i;
 	case WFT_STD:
 		break;
 	case WFT_PLOT:
-		wf->pixels = d_alloc(env.win_width * env.win_height 
+		wf->pixels = d_alloc(gf->win_width * gf->win_height 
 				* sizeof(long));
-		for ( i = 0 ; i < env.win_width * env.win_height ; i ++ )
+		for ( i = 0 ; i < gf->win_width * gf->win_height ; i ++ )
 			wf->pixels[i] = C_TRANSPARENT|C_DIRTY;
 		break;
 	default:
@@ -435,13 +440,15 @@ int i;
 
 
 void
-copy_pixels(WIN_FLAME * wf,unsigned long * buf,REAL1 * reso,GB_RECT * r)
+copy_pixels(
+	GBVIEW_FLAME * gf,
+	WIN_FLAME * wf,unsigned long * buf,REAL1 * reso,GB_RECT * r)
 {
 int x,y,i,index;
 int xx,yy;
 unsigned long * dest;
 int f;
-extern void _wf_set_redraw();
+
 
 	if ( wf->mov_flag )
 		return;
@@ -450,15 +457,15 @@ extern void _wf_set_redraw();
 	r->br.y += wf->mov_y;
 	r->br.x += wf->mov_x;
 	i = 0;
-	insert_redraw_gbr(r);
+	insert_redraw_gbr(gf,r);
 	f = 0;
 	for ( y = r->tl.y ; y < r->br.y ; y ++ )
 		for ( x = r->tl.x ; x < r->br.x ; x ++ ) {
-			if ( x < 0 || x >= env.win_width ) {
+			if ( x < 0 || x >= gf->win_width ) {
 				i ++;
 				continue;
 			}
-			if ( y < 0 || y >= env.win_height ) {
+			if ( y < 0 || y >= gf->win_height ) {
 				i ++;
 				continue;
 			}
@@ -466,21 +473,23 @@ extern void _wf_set_redraw();
 				i ++;
 				continue;
 			}
-			index = x + y*env.win_width;
+			index = x + y*gf->win_width;
 			dest = &wf->pixels[index];
 			*dest = buf[i]|(*dest&C_DIRTY);
-			env.redraw[index] |= RP_REDRAW;
+			gf->redraw[index] |= RP_REDRAW;
 			i ++;
 			f = 1;
 		}
 	if ( f )
-		new_tick(_wf_set_redraw,0,0);
+		new_tick(_wf_set_redraw,0,(int)gf);
 }
 
 
 
 int
-e2d_redraw_rect(WIN_FLAME * wf,GB_RECT *r,int * pri_cnt)
+e2d_redraw_rect(
+	GBVIEW_FLAME * gf,
+	WIN_FLAME * wf,GB_RECT *r,int * pri_cnt)
 {
 int min_x,max_x;
 int min_y,max_y;
@@ -501,28 +510,28 @@ int ci_ret;
 			return -1;
 		r->tl.x = 0;
 	}
-	if ( r->tl.x >= env.win_width )
+	if ( r->tl.x >= gf->win_width )
 		return -1;
 	if ( r->br.x <= 0 )
 		return -1;
-	if ( r->br.x > env.win_width ) {
-		if ( r->tl.x >= env.win_width )
+	if ( r->br.x > gf->win_width ) {
+		if ( r->tl.x >= gf->win_width )
 			return -1;
-		r->br.x = env.win_width;
+		r->br.x = gf->win_width;
 	}
 	if ( r->tl.y < 0 ) {
 		if ( r->br.y <= 0 )
 			return -1;
 		r->tl.y = 0;
 	}
-	if ( r->tl.y >= env.win_height )
+	if ( r->tl.y >= gf->win_height )
 		return -1;
 	if ( r->br.y <= 0 )
 		return -1;
-	if ( r->br.y > env.win_height ) {
-		if ( r->tl.y >= env.win_height )
+	if ( r->br.y > gf->win_height ) {
+		if ( r->tl.y >= gf->win_height )
 			return -1;
-		r->br.y = env.win_height;
+		r->br.y = gf->win_height;
 	}
 	min_x = r->br.x;
 	min_y = r->br.y;
@@ -530,21 +539,17 @@ int ci_ret;
 	max_y = r->tl.y;
 
 
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 
 
-	if ( _wf_redraw_pri(wf) < 0 ) {
-		unlock_task(env.wf_lock,"wf_redraw_rect");
-		return 0;
-	}
 
-	if ( _check_exit_flame() ) {
-		unlock_task(env.wf_lock,"wf_redraw_rect");
+	if ( _check_exit_flame(gf) ) {
+		wf_unlock(gf);
 		return -3;
 	}
 
 	if ( wf->flags & WFF_RESIZE ) {
-		unlock_task(env.wf_lock,"wf_redraw_rect");
+		wf_unlock(gf);
 		return -2;
 	}
 	for ( td = wf->next ; td ; td = td->next )
@@ -552,11 +557,11 @@ int ci_ret;
 			break;
 	for ( xx = r->tl.x ; xx < r->br.x ; xx ++ )
 		for ( yy = r->tl.y ; yy < r->br.y ; yy ++ ) {
-			if ( !(wf->pixels[yy*env.win_width + xx]&C_DIRTY) )
+			if ( !(wf->pixels[yy*gf->win_width + xx]&C_DIRTY) )
 				continue;
 /*
 			if ( td &&
-				!(td->pixels[yy*env.win_width + xx]
+				!(td->pixels[yy*gf->win_width + xx]
 					& C_TRANSPARENT) )
 				continue;
 */
@@ -570,7 +575,7 @@ int ci_ret;
 				max_y = yy+1;
 		}
 	if ( max_x < min_x ) {
-		unlock_task(env.wf_lock,"wf_redraw_rect");
+		wf_unlock(gf);
 		return -1;
 	}
 	r->tl.x = min_x;
@@ -594,40 +599,41 @@ int ci_ret;
 		for ( xx = r->tl.x ; xx < r->br.x ; xx ++ ) {
 			pt_list[i].x = xx;
 			pt_list[i].y = yy;
-			if ( !(wf->pixels[yy*env.win_width + xx]&
+			if ( !(wf->pixels[yy*gf->win_width + xx]&
 					C_DIRTY) ) {
 				pt_reso[i] = -1;
 				i ++;
 				continue;
 			}
-			wf->pixels[yy*env.win_width + xx]
+			wf->pixels[yy*gf->win_width + xx]
 				&= ~C_DIRTY;
-			pt_reso[i] = env.flame_base_resolution;
+			pt_reso[i] = gf->flame_base_resolution;
 			buf[i] = C_TRANSPARENT;
 			i ++;	
 		}	
 
 
-	mh = add_mh(&env.flame_base_display_map,wf->mh);
+	mh = add_mh(&gf->flame_base_display_map,wf->mh);
 
 	if ( wf->flags & WFF_LUSTER )
 		map_from_flame(mh,pt_list,pt_reso,w*h);
 
-
-	set_surp(&surp,r,env.flame_base_resolution);
+	set_surp(&surp,r,gf->flame_base_resolution);
 	map_from_flame(mh,surp.ptr,surp.reso,SURP_NOS);
 	id = wf->id;
-	unlock_task(env.wf_lock,"wf_redraw_rect");
+	wf_unlock(gf);
 
 
-	ci_ret = cache_image(wf->id,wf->draw,pt_list,pt_reso,buf,
+	ci_ret = cache_image(
+		gf,
+		wf->id,wf->draw,pt_list,pt_reso,buf,
 		w*h,r,mh,&surp,1,
 		WFF_PLOT_DIRTY|WFF_LUSTER_DIRTY);
 
 
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 set_t_msg(58);
-	_wf = _get_wf_ptr(id);
+	_wf = _get_wf_ptr(gf,id);
 	if ( _wf != wf )
 		goto end;
 
@@ -639,16 +645,16 @@ set_t_msg(58);
 		d_f_ree(pt_list);
 		d_f_ree(pt_reso);
 		free_mh(mh);
-		unlock_task(env.wf_lock,"wf_redoraw_rect");
+		wf_unlock(gf);
 		return -2;
 	}
-	copy_pixels(wf,buf,pt_reso,r);
+	copy_pixels(gf,wf,buf,pt_reso,r);
 	if ( wf->mov_flag )
 		wf_free_si(wf);
 	else	regulate_si(wf);
 
 end:
-	unlock_task(env.wf_lock,"wf_redoraw_rect");
+	wf_unlock(gf);
 	d_f_ree(buf);
 	d_f_ree(pt_list);
 	d_f_ree(pt_reso);
@@ -658,8 +664,6 @@ end:
 
 
 
-extern int interface_flag;
-
 int spiral_count;
 
 typedef struct redraw_pixel {
@@ -671,18 +675,17 @@ typedef struct redraw_pixel {
 	int		delta[4];
 } REDRAW_PIXEL;
 
-REDRAW_PIXEL rp;
 
 void
-set_redraw_pixel()
+set_redraw_pixel(REDRAW_PIXEL*rp,GBVIEW_FLAME * gf)
 {
 double unit;
 	unit = sqrt(((double)DL_PITCH)*DL_PITCH/
-		    (env.win_width*env.win_height));
-	rp.w = unit*env.win_width;
-	rp.h = unit*env.win_height;
-	rp.st_w = env.win_width/2 - rp.w/2;
-	rp.st_h = env.win_height/2 - rp.h/2;
+		    (gf->win_width*gf->win_height));
+	rp->w = unit*gf->win_width;
+	rp->h = unit*gf->win_height;
+	rp->st_w = gf->win_width/2 - rp->w/2;
+	rp->st_h = gf->win_height/2 - rp->h/2;
 }
 
 int
@@ -714,7 +717,7 @@ home:
 }
 
 int
-_e2d_redraw(WIN_FLAME * wf)
+_e2d_redraw(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 int xx,yy;
 GB_RECT rr;
@@ -723,15 +726,17 @@ int dx,dy;
 int ret;
 int sc;
 int pri_cnt;
+REDRAW_PIXEL rp;
+
 
 
 restart:
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 set_t_msg(59);
 	sc = 0;
 	wf->flags &= ~WFF_RESIZE;
-	set_redraw_pixel();
-	unlock_task(env.wf_lock,"load_spiral");
+	set_redraw_pixel(&rp,gf);
+	wf_unlock(gf);
 	xx = yy = 0;
 	t = l = r = b = 0;
 	ret = 0;
@@ -739,11 +744,11 @@ set_t_msg(59);
 	for ( ; ; ) {
 		if ( rp.st_h + yy*rp.h < 0 )
 			t = 1;
-		if ( rp.st_h + yy*rp.h >= env.win_height )
+		if ( rp.st_h + yy*rp.h >= gf->win_height )
 			b = 1;
 		if ( rp.st_w + xx*rp.w < 0 )
 			l = 1;
-		if ( rp.st_w + xx*rp.w >= env.win_width )
+		if ( rp.st_w + xx*rp.w >= gf->win_width )
 			r = 1;
 		rr.tl.x = rp.st_w + xx*rp.w;
 		rr.tl.y = rp.st_h + yy*rp.h;
@@ -754,7 +759,7 @@ printf("** %i(%x) ** xx %i %i %i %i %i %i\n",get_tid(),wf->flags,
 xx,yy,t,b,l,r);
 */
 
-		switch ( e2d_redraw_rect(wf,&rr,&pri_cnt) ) {
+		switch ( e2d_redraw_rect(gf,wf,&rr,&pri_cnt) ) {
 		case 0:
 			ret = 1;
 		case -1:
@@ -780,65 +785,59 @@ xx,yy,t,b,l,r);
 	spiral_count = sc;
 end:
 	if ( spiral_count )
-		_set_pri_adj(wf,pri_cnt*PRI_OFFSET_RATE/spiral_count);
-	else	_set_pri_adj(wf,pri_cnt);
+		_set_pri_adj(gf,wf,pri_cnt*PRI_OFFSET_RATE/spiral_count);
+	else	_set_pri_adj(gf,wf,pri_cnt);
 
 	return ret;
 }
 
 int
-e2d_redraw(WIN_FLAME * wf)
+e2d_redraw(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 int ret;
 int flags;
 	ret = 0;
 
-	lock_task(env.wf_lock);
+	wf_lock(gf);
 set_t_msg(60);
 	set_weight(wf);
 	flags = wf->flags;
 	if ( wf->flags & (WFF_POLY_DIRTY|WFF_LUSTER_DIRTY) ) {
 		wf->flags &= ~(WFF_POLY_DIRTY|WFF_LUSTER_DIRTY);
-		unlock_task(env.wf_lock,"e2d_redraw");
-		if ( _e2d_redraw(wf) == 2 ) {
+		wf_unlock(gf);
+		if ( _e2d_redraw(gf,wf) == 2 ) {
 			ret = 2;
 			wf->flags |= flags &
 				(WFF_POLY_DIRTY|WFF_LUSTER_DIRTY);
 		}
 		else	ret = 1;
 	}
-	else	unlock_task(env.wf_lock,"e2d_redraw");
-	lock_task(env.wf_lock);
+	else	wf_unlock(gf);
+	wf_lock(gf);
 set_t_msg(61);
 	if ( wf->flags & WFF_PLOT_DIRTY ) {
 		wf->flags &= ~(WFF_PLOT_DIRTY|WFF_RESIZE);
-		if ( _wf_redraw_pri(wf) < 0 ) {
-			ret = 0;
-			unlock_task(env.wf_lock,"e2d_redraw");
-		}
-		else {
-			unlock_task(env.wf_lock,"e2d_redraw");
-			if ( _e2d_p_redraw(wf) == 2 ) {
-				ret = 2;
-				wf->flags |= flags &
-					(WFF_PLOT_DIRTY|WFF_RESIZE);
-			}
-			else	ret = 1;
+		wf_unlock(gf);
+		if ( _e2d_p_redraw(gf,wf) == 2 ) {
+			ret = 2;
+			wf->flags |= flags &
+				(WFF_PLOT_DIRTY|WFF_RESIZE);
 		}
+		else	ret = 1;
 	}
-	else	unlock_task(env.wf_lock,"e2d_redraw");
+	else	wf_unlock(gf);
 	return ret;
 }
 
 int
-e2d_dirty(WIN_FLAME * wf,GB_RECT * r,int flags,int mode)
+e2d_dirty(GBVIEW_FLAME * gf,WIN_FLAME * wf,GB_RECT * r,int flags,int mode)
 {
 int i;
 int x,y;
 VRECT rr;
 unsigned long * p;
 	if ( r == 0 ) {
-		for ( i = 0; i < env.win_width*env.win_height ; i ++ )
+		for ( i = 0; i < gf->win_width*gf->win_height ; i ++ )
 			wf->pixels[i] |= C_DIRTY;
 	}
 	else {
@@ -848,15 +847,15 @@ unsigned long * p;
 		if ( r->tl.y < 0 )
 			rr.tl.y = 0;
 		else	rr.tl.y = r->tl.y;
-		if ( r->br.x >= env.win_width )
-			rr.br.x = env.win_width;
+		if ( r->br.x >= gf->win_width )
+			rr.br.x = gf->win_width;
 		else	rr.br.x = r->br.x;
-		if ( r->br.y >= env.win_height )
-			rr.br.y = env.win_height;
+		if ( r->br.y >= gf->win_height )
+			rr.br.y = gf->win_height;
 		else	rr.br.y = r->br.y;
 
 		for ( y = rr.tl.y ; y < rr.br.y ; y ++ ) {
-			p = &wf->pixels[y*env.win_width + rr.tl.x];
+			p = &wf->pixels[y*gf->win_width + rr.tl.x];
 			for ( x = rr.tl.x ; x < rr.br.x ; x ++ , p ++ )
 				*p |= C_DIRTY;
 		}
@@ -865,12 +864,12 @@ unsigned long * p;
 	wf->rd_mode = mode;
 */
 	wf->flags |= flags & WFF_DIRTY;
-	_wakeup_wf(wf);
+	_wakeup_wf(gf,wf);
 	return 0;
 }
 
 int
-e2d_overlay(WIN_FLAME * wf,OV_ARG * a)
+e2d_overlay(GBVIEW_FLAME * gf,WIN_FLAME * wf,OV_ARG * a)
 {
 	a->src = wf->pixels;
 	a->s_x = a->d_x;
@@ -883,27 +882,26 @@ e2d_overlay(WIN_FLAME * wf,OV_ARG * a)
 }
 
 int
-e2d_zoom(WIN_FLAME * wf,int w,int h,int * ix_x,int * ix_y)
+e2d_zoom(GBVIEW_FLAME * gf,WIN_FLAME * wf,int w,int h,int * ix_x,int * ix_y)
 {
-extern int null_wf_spiral;
 int i;
 
 	wf_free_si(wf);
 	wf_im_zoom_proc(wf->pixels,w,h,ix_x,ix_y);
-	for ( i = 0 ; i < env.win_height*env.win_width ; i ++ )
+	for ( i = 0 ; i < gf->win_height*gf->win_width ; i ++ )
 		wf->pixels[i] |= C_DIRTY;
 	wf->flags |= WFF_DIRTY;
 	return 0;
 }
 
 int
-e2d_resize(WIN_FLAME * wf)
+e2d_resize(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 int w,h;
 int i,x,y,dx,dy;
 int w2,h2;
-	w = env.win_width;
-	h = env.win_height;
+	w = gf->win_width;
+	h = gf->win_height;
 
 	wf->pixels = d_re_alloc(wf->pixels,w*h*sizeof(long));
 	for ( i = 0 ; i < w*h ; i ++ )
@@ -919,17 +917,14 @@ int w2,h2;
 
 
 int
-e2d_rotate(WIN_FLAME * wf,VPOINT * ix)
+e2d_rotate(GBVIEW_FLAME * gf,WIN_FLAME * wf,VPOINT * ix)
 {
 unsigned long * plane, * pp;
 int i;
-unsigned long * _rotate_plane(
-	unsigned long * plane,
-	VPOINT * index);
 
-	plane = _rotate_plane(wf->pixels,ix);
+	plane = _rotate_plane(gf,wf->pixels,ix);
 	pp = plane;
-	for ( i = 0 ; i < env.win_width * env.win_height ; i ++ )
+	for ( i = 0 ; i < gf->win_width * gf->win_height ; i ++ )
 		*pp ++ |= C_DIRTY;
 	d_f_ree(wf->pixels);
 	wf->pixels = plane;
