@@ -205,7 +205,7 @@ set_cpu_msg(720030);
 	if ( check_map_ptr(&n->mp_tbl[pos+1]) )
 		d_f_ree(n->mp_tbl[pos+1].map);
 set_cpu_msg(720031);
-	n->mp_tbl[pos+1].map = ll_copy_str(result->map,346);
+	n->mp_tbl[pos+1].map = ll_copy_str(result->map);
 	n->mp_ptr = j;
 	free_around_list(a);
 set_cpu_msg(720032);
@@ -231,10 +231,10 @@ set_cpu_msg(720003);
 set_cpu_msg(720004);
 		if ( n->target ) {
 		URL u1,u2;
-			get_url2(&u1,n->target,1248);
+			get_url2(&u1,n->target);
 			if ( n->mp_ptr )
-				get_url2(&u2,n->mp_tbl[n->mp_ptr-1].crd,1249);
-			else 	get_url2(&u2,n->start,1250);
+				get_url2(&u2,n->mp_tbl[n->mp_ptr-1].crd);
+			else 	get_url2(&u2,n->start);
 			if ( url_cmp(&u1,&u2) ) {
 				free_url(&u1);
 				free_url(&u2);
@@ -297,7 +297,7 @@ L_CHAR * f;
 URL u;
 void gc_gb_sexp();
 
-	get_url2(&u,target,1609);
+	get_url2(&u,target);
 
 	gc_push(0,0,"remote_fetch");
 	gt = get_symbol(l_string(std_cm,"Get"));
@@ -367,7 +367,7 @@ int local_ip,p;
 			break;
 		}
 
-		get_url2(&u,n->target,346);
+		get_url2(&u,n->target);
 		if ( cmp_site(u.server,0,u.port,
 				0,local_ip,p) == 0 ) {
 			n->target_cid = url2cid(n->target);
@@ -521,8 +521,8 @@ URL u;
 			if ( n->h.key )
 				d_f_ree(n->h.key);
 			if ( n->mp_ptr == 0 )
-				get_url2(&u,n->start,346);
-			else	get_url2(&u,n->mp_tbl[n->mp_ptr-1].crd,346);
+				get_url2(&u,n->start);
+			else	get_url2(&u,n->mp_tbl[n->mp_ptr-1].crd);
 			n->h.key = get_server_key(&u);
 			free_url(&u);
 			insert_queue_mp_t(&rque3,n,1);
@@ -591,7 +591,7 @@ URL u;
 	set_attribute(gt,
 		l_string(std_cm,"ttl"),
 		l_string(std_cm,buf));
-	get_url2(&u,url,246);
+	get_url2(&u,url);
 	ret = remote_session(
 		gblisp_top_env0,
 		ses,
@@ -626,7 +626,7 @@ XL_SEXP * t;
 		if ( get_type(s) != XLT_STRING )
 			return MPE_UNKNOWN;
 		mp->map = 0;
-		mp->crd = ll_copy_str(s->string.data,1381);
+		mp->crd = ll_copy_str(s->string.data);
 		return 1;
 	}
 	else {
@@ -639,7 +639,7 @@ set_cpu_msg(720021);
 			t = car(s);
 			if ( get_type(t) != XLT_STRING )
 				return MPE_UNKNOWN;
-			mp[mp_ptr].map = ll_copy_str(t->string.data,1380);
+			mp[mp_ptr].map = ll_copy_str(t->string.data);
 			s = cdr(s);
 			t = car(s);
 set_cpu_msg(720022);
@@ -648,7 +648,7 @@ set_cpu_msg(720022);
 				mp[mp_ptr].map = 0;
 				return MPE_UNKNOWN;
 			}
-			mp[mp_ptr].crd = ll_copy_str(t->string.data,1379);
+			mp[mp_ptr].crd = ll_copy_str(t->string.data);
 			s= cdr(s);
 set_cpu_msg(720023);
 		}
@@ -904,8 +904,8 @@ set_cpu_msg(721002);
 		case XLT_STRING:
 set_cpu_msg(721102);
 			ret_sexp = new_d_sexp(&n->d);
-			n->target = ll_copy_str(target->string.data,346);
-			get_url2(&u,target->string.data,365);
+			n->target = ll_copy_str(target->string.data);
+			get_url2(&u,target->string.data);
 			n->h.key = get_server_key(&u);
 			free_url(&u);
 			insert_queue_mp_t(&rque1,n,1);
@@ -945,7 +945,7 @@ set_cpu_msg(721007);
 set_cpu_msg(721008);
 	if ( _target == 0 )
 		goto invalid_param2;
-	mp = d_alloc(sizeof(MAP_PATH)*ttl,23);
+	mp = d_alloc(sizeof(MAP_PATH)*ttl);
 set_cpu_msg(720002);
 	reset_mp(mp,ttl);
 	ret = MP_routing_over_site(&len,mp,ttl,
@@ -956,10 +956,10 @@ set_cpu_msg(720003);
 set_cpu_msg(720004);
 		if ( get_type(target) == XLT_STRING ) {
 		URL u1,u2;
-			get_url2(&u1,target->string.data,1248);
+			get_url2(&u1,target->string.data);
 			if ( len )
-				get_url2(&u2,mp[len-1].crd,1249);
-			else 	get_url2(&u2,from->string.data,1250);
+				get_url2(&u2,mp[len-1].crd);
+			else 	get_url2(&u2,from->string.data);
 			if ( url_cmp(&u1,&u2) ) {
 				free_url(&u1);
 				free_url(&u2);
