@@ -20,6 +20,7 @@
 #include "utils.h"
 #include "gbview.h"
 #include "xlerror.h"
+#include "win_flame.h"
 
 XL_SEXP * xl_gv_flame_status();
 
@@ -39,7 +40,11 @@ xl_gv_flame_status(
 {
 XL_SEXP *ret;
 GBVIEW_STATUS gs;
-	wf_status(&gs);
+GBVIEW_FLAME * gf;
+
+	gf = get_gf_from_sf(sf);
+
+	wf_status(gf,&gs);
 	ret = gvstatus2list(&gs);
 	wf_free_status(&gs);
 	return ret;
