@@ -20,6 +20,8 @@
 #ifndef ___PFC_H___
 #define ___PFC_H___
 
+#include	"xl.h"
+
 typedef struct pfc {
 	char		filename[100];
 	int		interval;
@@ -28,7 +30,7 @@ typedef struct pfc {
 	double		evaluation;
 	unsigned int	last_checkpoint;
 	unsigned int	accumulate;
-	int		lock_id;
+	CALL_LOCK_DESCRIPTER lock_id;
 } PFC;
 
 #define INTERVAL_MIN	10
