@@ -320,6 +320,7 @@ typedef struct res_header {
 #define RF_FIX			0x00000004
 
 #define RF_CACHE		0x00000010
+#define RF_UNCACHE		0x00000020
 
 #define RF_COMPLETE		0x00000100
 #define RF_LOAD_EXTEND		0x00000200
@@ -908,6 +909,8 @@ int xx_lock_pdb_lock(RESOURCE * r,int es,char * file,int line);
 void unlock_pdb_lock(RESOURCE * r);
 void pdb_err_sensitive(RESOURCE *,int);
 
+COORDINATE_UNIT *
+set_c_unit(COORDINATE_UNIT *,L_CHAR * url,L_CHAR * system,L_CHAR * unit);
 void setup_c_unit(COORDINATE_UNIT * up,int ses,L_CHAR * url,
 	XL_SEXP * meta,XL_SYM_FIELD * sf);
 COORDINATE_UNIT * search_c_unit(L_CHAR * url);
