@@ -262,11 +262,11 @@ static PDB_PD_POINT *point_array_to_pd_point_list(GB_POINT* src_array, int src_c
 	int i;
 	for(i=0; i<src_count; ++i){
 		if(ret==0){
-			ret = (PDB_PD_POINT*)d_calloc(sizeof(PDB_PD_POINT), 1, 456);
+			ret = (PDB_PD_POINT*)d_calloc(sizeof(PDB_PD_POINT), 1);
 			dest_ptr = ret;
 		}
 		else{
-			dest_ptr->next = (PDB_PD_POINT*)d_calloc(sizeof(PDB_PD_POINT), 1, 456);
+			dest_ptr->next = (PDB_PD_POINT*)d_calloc(sizeof(PDB_PD_POINT), 1);
 			dest_ptr = dest_ptr->next;
 		}
 		dest_ptr->reso = -1;
