@@ -1,4 +1,4 @@
-/* ver.A.04(ASTROBOY) */
+/* ver.A.04.01(ASTROBOY) */
 
 
 #include "version.h"
@@ -8,8 +8,8 @@ VERSION ap_version = {
 	"AP.code",
 	"cosmos",
 	"(C) 2000-2004 Hirohisa Mori And GB-group",
-	"ver.A.04(ASTROBOY)",
-	2004,3,12,
+	"ver.A.04.01(ASTROBOY)",
+	2004,3,24,
 	0,
 	"GLOBALBASE client ",
 };
