@@ -1,4 +1,4 @@
-/* ver.A.03.08(ASTROBOY) */
+/* ver.A.04(ASTROBOY) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"pdbstatus",
-	"(C) 2000-2004 Hirohisa Mori",
-	"ver.A.03.08(ASTROBOY)",
-	2004,2,5,
+	"(C) 2000-2004 Hirohisa Mori And GB-group",
+	"ver.A.04(ASTROBOY)",
+	2004,2,19,
 	0,
 	"pdb status checker ",
 };
