@@ -899,7 +899,7 @@ CS_LIST * target;
 		type = ch&get_lc_mask(ch);
 		target = search_cs(w->lst,type);
 		if ( target == 0 ) {
-			target = d_alloc(sizeof(*target),2);
+			target = d_alloc(sizeof(*target));
 			target->next = w->lst;
 			w->lst = target;
 			target->code = type;
